@@ -622,6 +622,16 @@ cd_device_connect_cb (GObject *source_object,
 	if (id != NULL)
 		device->priv->id = g_variant_dup_string (id, NULL);
 
+	/* if the device is missing, then fail */
+	if (id == NULL) {
+		g_simple_async_result_set_error (res_source,
+						 CD_DEVICE_ERROR,
+						 CD_DEVICE_ERROR_FAILED,
+						 "Failed to connect to missing device %s",
+						 cd_device_get_object_path (device));
+		goto out;
+	}
+
 	/* get kind */
 	kind = g_dbus_proxy_get_cached_property (device->priv->proxy,
 						 CD_DEVICE_PROPERTY_KIND);
@@ -762,6 +772,7 @@ cd_device_connect (CdDevice *device,
 	GSimpleAsyncResult *res;
 
 	g_return_if_fail (CD_IS_DEVICE (device));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
 	res = g_simple_async_result_new (G_OBJECT (device),
 					 callback,
@@ -788,8 +799,6 @@ cd_device_connect (CdDevice *device,
 
 /**********************************************************************/
 
-/**********************************************************************/
-
 /**
  * cd_device_set_property_finish:
  * @device: a #CdDevice instance.
@@ -874,6 +883,9 @@ cd_device_set_property (CdDevice *device,
 	GSimpleAsyncResult *res;
 
 	g_return_if_fail (CD_IS_DEVICE (device));
+	g_return_if_fail (key != NULL);
+	g_return_if_fail (value != NULL);
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (device->priv->proxy != NULL);
 
 	res = g_simple_async_result_new (G_OBJECT (device),
@@ -978,6 +990,7 @@ cd_device_add_profile (CdDevice *device,
 
 	g_return_if_fail (CD_IS_DEVICE (device));
 	g_return_if_fail (CD_IS_PROFILE (profile));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (device->priv->proxy != NULL);
 
 	res = g_simple_async_result_new (G_OBJECT (device),
@@ -1080,6 +1093,8 @@ cd_device_remove_profile (CdDevice *device,
 	GSimpleAsyncResult *res;
 
 	g_return_if_fail (CD_IS_DEVICE (device));
+	g_return_if_fail (CD_IS_PROFILE (profile));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (device->priv->proxy != NULL);
 
 	res = g_simple_async_result_new (G_OBJECT (device),
@@ -1181,6 +1196,8 @@ cd_device_make_profile_default (CdDevice *device,
 	GSimpleAsyncResult *res;
 
 	g_return_if_fail (CD_IS_DEVICE (device));
+	g_return_if_fail (CD_IS_PROFILE (profile));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (device->priv->proxy != NULL);
 
 	res = g_simple_async_result_new (G_OBJECT (device),
@@ -1281,6 +1298,7 @@ cd_device_profiling_inhibit (CdDevice *device,
 	GSimpleAsyncResult *res;
 
 	g_return_if_fail (CD_IS_DEVICE (device));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (device->priv->proxy != NULL);
 
 	res = g_simple_async_result_new (G_OBJECT (device),
@@ -1380,6 +1398,7 @@ cd_device_profiling_uninhibit (CdDevice *device,
 	GSimpleAsyncResult *res;
 
 	g_return_if_fail (CD_IS_DEVICE (device));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (device->priv->proxy != NULL);
 
 	res = g_simple_async_result_new (G_OBJECT (device),
@@ -1493,6 +1512,8 @@ cd_device_get_profile_for_qualifiers (CdDevice *device,
 	GVariantBuilder builder;
 
 	g_return_if_fail (CD_IS_DEVICE (device));
+	g_return_if_fail (qualifiers != NULL);
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (device->priv->proxy != NULL);
 
 	/* squash char** into an array of strings */
@@ -1606,6 +1627,8 @@ cd_device_get_profile_relation (CdDevice *device,
 	GSimpleAsyncResult *res;
 
 	g_return_if_fail (CD_IS_DEVICE (device));
+	g_return_if_fail (CD_IS_PROFILE (profile));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (device->priv->proxy != NULL);
 
 	res = g_simple_async_result_new (G_OBJECT (device),
