@@ -585,6 +585,16 @@ cd_profile_connect_cb (GObject *source_object,
 	if (id != NULL)
 		profile->priv->id = g_variant_dup_string (id, NULL);
 
+	/* if the profile is missing, then fail */
+	if (id == NULL) {
+		g_simple_async_result_set_error (res_source,
+						 CD_PROFILE_ERROR,
+						 CD_PROFILE_ERROR_FAILED,
+						 "Failed to connect to missing profile %s",
+						 cd_profile_get_object_path (profile));
+		goto out;
+	}
+
 	/* get filename */
 	filename = g_dbus_proxy_get_cached_property (profile->priv->proxy,
 						     CD_PROFILE_PROPERTY_FILENAME);
@@ -716,6 +726,7 @@ cd_profile_connect (CdProfile *profile,
 	GSimpleAsyncResult *res;
 
 	g_return_if_fail (CD_IS_PROFILE (profile));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
 	res = g_simple_async_result_new (G_OBJECT (profile),
 					 callback,
@@ -826,6 +837,9 @@ cd_profile_set_property (CdProfile *profile,
 	GSimpleAsyncResult *res;
 
 	g_return_if_fail (CD_IS_PROFILE (profile));
+	g_return_if_fail (key != NULL);
+	g_return_if_fail (value != NULL);
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (profile->priv->proxy != NULL);
 
 	res = g_simple_async_result_new (G_OBJECT (profile),
@@ -926,6 +940,7 @@ cd_profile_install_system_wide (CdProfile *profile,
 	GSimpleAsyncResult *res;
 
 	g_return_if_fail (CD_IS_PROFILE (profile));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (profile->priv->proxy != NULL);
 
 	res = g_simple_async_result_new (G_OBJECT (profile),
